#!/usr/bin/python3

import sys
import os
import subprocess
import re
import shutil
from glob import glob
import xml.etree.ElementTree as ET
from exceptions import exceptions

ask = input("Start processing? ")
if ask.lower() in ["yes", "y"]:
    pass
else:
    sys.exit(0)

deb_files = glob("*.deb")

xml = open("catalog.xml", "a")
#txt = open("catalog.txt", "w")

xml.write("<catalog>")

directory = os.getcwd()
os.chdir(directory)

def cleaner(string):
    string = string.replace("&", "and")
    string = string.replace("<", "").replace(">", "")
    string = string.replace("\"", "'")
    return string

def get_file_size(file_path):
    size = subprocess.check_output("du -sh \"{}\"".format(file_path), shell=True).split()[0].decode('utf-8')
    
    size = size.replace(",", "").replace(".0", "")
    
    if size.endswith("K"):
        size = size.replace("K", " KB")
    elif size.endswith("M"):
        size = size.replace("M", " MB")
    elif size.endswith("G"):
        size = size.replace("G", " GB")
    return size

for deb_file in deb_files:
    
    try:

        deb_name = deb_file.replace(".deb", "")

        control_dir = os.path.join(deb_name, "DEBIAN")

        os.makedirs(deb_name, exist_ok=True)

        subprocess.call(f"dpkg-deb -e \"{deb_file}\" \"{control_dir}\"", shell=True)

        with open(os.path.join(control_dir, "control"), "r") as f:
            for line in f.readlines():
                match = re.search(r'^Version:\s*(.+)', line)
                if match:
                    version = cleaner(match.group(1))

                match = re.search(r'^Package:\s*(.+)', line)
                if match:
                    package = cleaner(match.group(1))
            
                match = re.search(r'^Maemo-Display-Name:\s*(.+)', line)
                if match:
                    display_name = cleaner(match.group(1))

                if package in exceptions.keys():
                    display_name = exceptions[package]

                match = re.search(r'^Maintainer:\s*(.+)', line)
                if match:
                    dev = match.group(1)
                    email = re.search(r'\s*<.*>', dev)
                    if email:
                        email = email.group(0)
                        dev = dev.replace(email, "")
                    dev = cleaner(dev)
                size = get_file_size(os.path.join(control_dir, "control"))

        try:
            if not display_name:
                display_name = package
        except NameError:
            display_name = package

        shutil.rmtree(deb_name)

        xml.write("""
    <app>
        <data deb="{}" name="{}" ver="{}" package="{}" dev="{}" size="{}" />
    </app>""".format(deb_file, display_name, version, package, dev, size))

        #txt.write(display_name + "\n")

        deb_file = None
        display_name = None
        version = None
        package = None
        dev = None
        size = None
        
    except Exception as e:
        exc_type, exc_obj, exc_tb = sys.exc_info()
        fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
        print(exc_type, fname, exc_tb.tb_lineno, deb_file)
        continue

xml.write("\n</catalog>")
xml.close()
#txt.close()
